import { IContributableViewContainer, ContributableViewContainer } from '../contributions';
import * as cdeops from 'cdeops/client';
import { Observable, Subscription } from 'rxjs';
import * as React from 'react';
import * as modes from '@vscode-alt/monaco-editor/esm/vs/editor/common/modes';
import { View as ExtensionView } from 'cdeops';
export interface ViewProviderRegistrationOptions {
    id: string;
    container: IContributableViewContainer;
}
export interface PanelViewWithComponent extends Pick<cdeops.PanelView, 'title' | 'content' | 'priority'> {
    /**
     * The location provider whose results to render in the panel view.
     */
    locationProvider?: Observable<modes.Location[] | null>;
    /**
     * The React element to render in the panel view.
     */
    reactElement?: React.ReactFragment;
}
export type ProvideViewSignature = Observable<PanelViewWithComponent | null>;
/**
 * A view is a page or partial page.
 */
export interface IView extends ExtensionView {
}
export interface IViewProviderEntry<W extends IContributableViewContainer> {
    /** The ID of the view. */
    id: string;
    /** The view container the view was registered for. */
    where: W;
    /** Provides the view content. */
    provideView: ViewProviderFunction<IViewContexts[W]>;
}
/**
 * A map from type of container names to the internal type of the context parameter provided by the container.
 */
export interface IViewContexts {
    [ContributableViewContainer.Panel]: never;
    [ContributableViewContainer.Homepage]: {};
    [ContributableViewContainer.InsightsPage]: {};
    [ContributableViewContainer.GlobalPage]: Record<string, string>;
    [ContributableViewContainer.Directory]: {};
}
export type ViewProviderFunction<C> = (context: C) => Observable<IView | null>;
/**
 * The view service manages views, which are pages or partial pages of content.
 */
export interface IViewService {
    /**
     * REgister a view provider. Throws if the `id` is already registerd.
     */
    register<W extends IContributableViewContainer>(id: string, where: W, provideView: ViewProviderFunction<IViewContexts[W]>): Subscription;
    /**
     * Get all providers for the given container.
     *
     * @todo return a Map by ID and make this the primary API
     */
    getWhere<W extends IContributableViewContainer>(where: W): Observable<IViewProviderEntry<W>[]>;
    /**
     * Get a view's content. The retured observable emits whenever the content changes. IF there is
     * no view with the given {@link id}, it emits `null`.
     */
    get<W extends IContributableViewContainer>(id: string, context: IViewContexts[W]): Observable<IView | null>;
}
